@@ -35,6 +35,18 @@ def main(cli, args):
     builder.archive_only = args.archive_only
     builder.archive_file = args.archive
 
+    if args.env:
+        env = {}
+        for entry in args.env:
+            parts = entry.split('=', 1)
+            if len(parts) != 2:
+                cli.err('Environment variables must follow the KEY=VALUE '
+                        'format. Invalid entry: "{0}".\n'.format(entry))
+                return 1
+            k, v = parts
+            env[k] = v
+        builder.extra_env = env
+
     if args.use_cache:
         cache_container = '{0}__buildcache'.format(steps.name)
         cache_hostpath = None
@@ -49,7 +61,7 @@ def main(cli, args):
             else:
                 cache_container, cache_volume = parts
             if not posixpath.isabs(cache_volume):
-                cli.err('The cache "path" must be an absolute path.')
+                cli.err('The cache "path" must be an absolute path.\n')
                 return 1
 
         builder.cache_container = cache_container
@@ -174,6 +186,7 @@ mkdir -p /root/.ssh
 chmod 700 /root/.ssh
 
 if [ -f ssh_identity ]; then
+    # copy instead of symlink so that we can chmod without issue
     cp ssh_identity /root/.ssh/ssh_identity
     chmod 600 /root/.ssh/ssh_identity
 fi
@@ -183,6 +196,7 @@ cat > /root/.ssh/config << EOF
     IdentityFile /root/.ssh/ssh_identity
 EOF
 
+# ensure we have a cache directory
 if [ ! -d "$BUILD_CACHE" ]; then
     mkdir -p "$BUILD_CACHE"
 fi
@@ -217,6 +231,8 @@ class DockerBuilder(object):
     archive_file = None
     archive_only = False
 
+    extra_env = None
+
     @staticmethod
     def stdout(msg):
         sys.stdout.write(msg)
@@ -231,6 +247,8 @@ class DockerBuilder(object):
     def run(self):
         self._setup()
         try:
+            if not self._get_image('busybox'):
+                return
             if not self._create_cache():
                 return
             if not self._build_source_container():
@@ -323,24 +341,51 @@ class DockerBuilder(object):
             log.info('found cache container=%s', self.cache_container)
         return True
 
+    def _get_image(self, name):
+        log.debug('checking for image=%s', name)
+        do_pull_image = False
+        try:
+            self.client.inspect_image(name)
+        except docker.errors.APIError as ex:
+            if ex.is_client_error():
+                do_pull_image = True
+                log.debug('could not find image=%s', name)
+            else:
+                raise
+
+        if do_pull_image:
+            log.info('pulling image=%s', name)
+            stream = self.client.pull(name, stream=True)
+            for chunk in stream:
+                self.stdout(chunk)
+        else:
+            log.info('found image=%s', name)
+        return True
+
     def _build_source_container(self):
         log.info('building source')
         self.archive_name = '%s-%s.tar.gz' % (
             self.steps.name, self.steps.version)
         self.archive_path = posixpath.join(self.dist_volume, self.archive_name)
 
+        env = {
+            'BUILD_ROOT': self.src_volume,
+            'BUILD_CONTEXT': posixpath.join(self.src_volume, 'context'),
+            'BUILD_ARCHIVE_PATH': self.archive_path,
+            'BUILD_NAME': self.steps.name,
+            'BUILD_VERSION': self.steps.version,
+            'BUILD_CACHE': self.cache_volume,
+        }
+        if self.extra_env:
+            env.update(self.extra_env)
+        for k in sorted(env.keys()):
+            log.info('builder env %s = %s', k, env[k])
+
         container = self.client.create_container(
             self.steps.compiler.base_image,
             command='/bin/bash build.sh',
             working_dir=self.src_volume,
-            environment={
-                'BUILD_ROOT': self.src_volume,
-                'BUILD_CONTEXT': posixpath.join(self.src_volume, 'context'),
-                'BUILD_ARCHIVE_PATH': self.archive_path,
-                'BUILD_NAME': self.steps.name,
-                'BUILD_VERSION': self.steps.version,
-                'BUILD_CACHE': self.cache_volume,
-            },
+            environment=env,
             volumes=[
                 self.src_volume,
                 self.dist_volume,
